{%- if cookiecutter.use_bitbucket == "yes" -%}
{%- set repo_host = 'BitBucket' -%}
{%- else -%}
{%- set repo_host = 'GitHub' -%}
{%- endif -%}
============
Contributing
============

Contributions are welcome, and they are greatly appreciated! Every
little bit helps, and credit will always be given.

You can contribute in many ways:

Types of Contributions
----------------------

Report Bugs
~~~~~~~~~~~

{% if cookiecutter.use_bitbucket == "yes" -%}
Report bugs at {{ cookiecutter.bitbucket_url }}/issues.
{%- else -%}
Report bugs at {{ cookiecutter.github_url }}/issues.
{%- endif %}

If you are reporting a bug, please include:

* Your operating system name and version.
* Any details about your local setup that might be helpful in troubleshooting.
* Detailed steps to reproduce the bug.

Fix Bugs
~~~~~~~~

Look through the {{ repo_host }} issues for bugs. Anything tagged with "bug"
is open to whoever wants to implement it.

Implement Features
~~~~~~~~~~~~~~~~~~

Look through the {{ repo_host }} issues for features. Anything tagged with
"feature" is open to whoever wants to implement it.

Write Documentation
~~~~~~~~~~~~~~~~~~~

{{ cookiecutter.project_name }} could always use more documentation, whether as
part of the official {{ cookiecutter.project_name }} docs, in docstrings, or
even on the web in blog posts, articles, and such.

Submit Feedback
~~~~~~~~~~~~~~~

The best way to send feedback is to file an issue at
{%- if cookiecutter.use_bitbucket == "yes" %}
{{ cookiecutter.bitbucket_url }}/issues.
{%- else %}
{{ cookiecutter.github_url }}/issues.
{%- endif %}

If you are proposing a feature:

* Explain in detail how it would work.
* Keep the scope as narrow as possible, to make it easier to implement.
* Remember that this is a volunteer-driven project, and that contributions
  are welcome :)

Get Started!
------------

Ready to contribute? Here's how to set up {{ cookiecutter.project_name }} for
local development.

1. Fork the `{{ cookiecutter.github_repo }}` repo on {{ repo_host }}.
2. Clone your fork locally::

{%- if cookiecutter.use_bitbucket == "yes" %}

    $ hg clone https://your_name_here@bitbucket.org/your_name_here/{{ cookiecutter.bitbucket_repo }}
{%- else %}

    $ git clone git@github.com:your_name_here/{{ cookiecutter.github_repo }}.git
{%- endif %}

3. Install your local copy into a virtualenv. Assuming you have
   virtualenvwrapper installed, this is how you set up your fork for local
   development::

    $ mkvirtualenv {{ cookiecutter.project_slug }}
    $ cd {{ cookiecutter.project_slug }}/
    $ python setup.py develop

4. Create a branch for local development::
{%- if cookiecutter.use_bitbucket == "yes" %}

    $ hg branch name-of-your-bugfix-or-feature
    $ hg up name-of-your-bugfix-or-feature
{%- else %}

    $ git checkout -b name-of-your-bugfix-or-feature
{%- endif %}

   Now you can make your changes locally.

5. When you're done making changes, check that your changes pass flake8 and the
   tests, including testing other Python versions with tox:

.. code-block:: bash

    $ make lint
    $ make test
    $ make test-all    # Optional, requires multiple versions of Python

To get flake8 and tox, just pip install them into your virtualenv.

6. Commit your changes and push your branch to {{ repo_host }}.::
{%- if cookiecutter.use_bitbucket == "yes" %}

    $ hg commit -m "Your detailed description of your changes."
    $ hg push 
{%- else %}

    $ git add .
    $ git commit -m "Your detailed description of your changes."
    $ git push origin name-of-your-bugfix-or-feature
{%- endif %}

7. Submit a pull request through the {{ repo_host }} website.


Pull Request Guidelines
-----------------------

Before you submit a pull request, check that it meets these guidelines:

1. The pull request should include tests.
2. If the pull request adds functionality, the docs should be updated. Put
   your new functionality into a function with a docstring, and add the
   feature to the list in README.rst.
3. The pull request should work for Python 2.7, and 3.3, 3.4, 3.5, and for PyPy.  Check
   https://travis-ci.org/{{ cookiecutter.github_username }}/{{ cookiecutter.github_repo }}/pulls
   and make sure that the tests pass for all supported Python versions.
